==> desktopbsd-tools-1.0.1.tar.bz2: desktopbsd-tools/libdesktopbsd/package.cpp <==
/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Authors: Peter Hofer <hofer.p@gmail.com>
 *			 Reinhard Pointner
 *			 (C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include "package.h"

#include <dirent.h>

#include <qfile.h>
#include <qregexp.h>

#include <pkgversion.h>
#include <executor.h>

Package::Package() {
	instStatus = Undefined;
	installedSizeSet = false;
	longDescrSet = false;
}

/** This method expects that it's package object is the
  * installed object (e.g. created by getInstalledPackages())
  * and the parameter is the object with the latest
  * information (INDEX-5).
  *
  * Calling this method when the package is not already marked
  * as Installed or InstalledOutdated fails.
  *
  */
void	Package::mergeWithLatestPackage(const Package & pkg) {
	if(instStatus != Installed  &&  instStatus != InstalledOutdated  &&  instStatus != InstalledNewer)
		return;

	setLatestVersion(pkg.getLatestVersion());

	int status = version_cmp(getInstalledVersion(), getLatestVersion());
	instStatus = (status == -1) ? InstalledOutdated : ((status == 0) ? Installed : InstalledNewer);

	/** TODO	Merge categories? */
}

/** Returns a list with all packages currently installed
  * on the system.
  */
QPtrList<Package>	Package::getInstalledPackages() {
	QPtrList<Package>	list;

	Package	*pkg;
	DIR		*dir = opendir(PKGDB);

	if(dir != NULL) {
		struct dirent	*ent;
		while((ent = readdir(dir)) != NULL)
			if(ent->d_type == DT_DIR  &&  strncmp(ent->d_name, ".", 1) != 0) {
				pkg = new Package();
				pkg->instStatus = Installed;
				pkg->installedName = ent->d_name;			/* e.g. cups-base-1.1.23.0_3	*/

				QStringList	pkgname = QStringList::split('-', ent->d_name, true);
				pkg->installedVersion = pkgname.last();	/* e.g. 1.1.23.0_3				*/
				pkgname.pop_back();
				pkg->name = pkgname.join("-");				/* e.g. cups-base					*/

				char  buffer[4096];
				char  fnbuffer[512] = { PKGDB };
				char  *end = stpcpy(fnbuffer + sizeof(PKGDB) - 1, ent->d_name);

				strcpy(end, "/+COMMENT");
				FILE  *fp = fopen(fnbuffer, "r");
				if(fp != NULL) {
					if(fgets(buffer, sizeof buffer, fp) != NULL)
						pkg->shortDescr = buffer;

					fclose(fp);
				}

				strcpy(end, "/+CONTENTS");
				fp = fopen(fnbuffer, "r");
				if(fp != NULL) {
					/* Note that we can only get the main category
					 * (by reading the ORIGIN comment)
					 */
					const char origin[] = { "@comment ORIGIN:" };
					char  *originBgn = buffer + sizeof origin - 1;
					while(fgets(buffer, sizeof buffer, fp) != NULL) {
						if(strncmp(buffer, origin, sizeof origin - 1) == 0) {
							/* Not used in UI
							 *
							pkg->portPath = QString(PORTSDIR) + originBgn;
							 */

							char  *slash = strchr(originBgn, '/');
							if(slash != NULL) {
								*slash = '\0';
								pkg->categories = originBgn;

								/* Remove '\n' retained by fgets() */
								char  *newline = strchr(slash + 1, '\n');
								if(newline != NULL)
									*newline = '\0';

								pkg->name = slash + 1;
							}

							break;
						}
					}

					fclose(fp);
				}

				strcpy(end, "/+DESC");
				fp = fopen(fnbuffer, "r");
				if(fp != NULL) {
					const char www[] = { "WWW: " };

					while(fgets(buffer, sizeof buffer, fp) != NULL)
						if(strncmp(buffer, www, sizeof www - 1) == 0) {
							char  *wwwStart = buffer + sizeof www - 1;
							while(*wwwStart == ' ')
								++wwwStart;

							if(*wwwStart != '\n'  &&  *wwwStart != '\0') {
								/* Remove '\n' retained by fgets() */
								char  *newline = strchr(buffer, '\n');
								if(newline != NULL)
									*newline = '\0';

								pkg->website = wwwStart;
							}

							break;
						}
					fclose(fp);
				}

				strcpy(end, "/+REQUIRED_BY");
				fp = fopen(fnbuffer, "r");
				if(fp != NULL) {
					while(fgets(buffer, sizeof buffer, fp) != NULL) {
						/* Remove '\n' retained by fgets() */
						char  *newline = strchr(buffer, '\n');
						if(newline != NULL)
							*newline = '\0';

						pkg->requiredBy.append(buffer);
					}

					fclose(fp);
				}

				list.append(pkg);
			}
	}
		  closedir(dir);

	return list;
}


/** Creates a "Package" object from an INDEX-5 line. The version
  * number from the file will be written into latestVersion.
  *
  * This function will not set requiredBy.
  */
Package  *Package::fromINDEXLine(const QString  & line) {
	 Package  *pkg = new Package();
	 QStringList  splitLine = QStringList::split('|', line, true);

	 /* Example:
	at-spi-1.6.1|/usr/ports/accessibility/at-spi|/usr/X11R6|An Assistive Technology
	Service Provider Interface|/usr/ports/accessibility/at-spi/pkg-descr|gnome@Fre
	eBSD.org|accessibility x11-toolkits|ORBit2-2.12.0 atk-1.8.0 bitstream-vera-1.10
	expat-1.95.8 fontconfig-2.2.3,1 freetype2-2.1.7_3 gail-1.8.1 gettext-0.13.1_1
	glib-2.4.8 gmake-3.80_2 gtk-2.4.14 hicolor-icon-theme-0.5 imake-6.7.0_2 intltoo
	l-0.32.1 jpeg-6b_3 libIDL-0.8.4 libXft-2.1.6 libart_lgpl2-2.3.16 libbonobo-2.8.
	0_1 libglade2-2.4.1 libgnomecanvas-2.8.0 libiconv-1.9.2_1 libtool-1.5.10 libxml
	2-2.6.16 p5-XML-Parser-2.34_1 pango-1.6.0 perl-5.8.5 pkgconfig-0.15.0_1 png-1.2
	.7 popt-1.7 python-2.4 shared-mime-info-0.15_1 tiff-3.7.0_2 xmlcatmgr-2.1 xorg-
	fonts-encodings-6.7.0 xorg-fonts-truetype-6.7.0 xorg-libraries-6.7.0_3|ORBit2-2
	.12.0 atk-1.8.0 bitstream-vera-1.10 expat-1.95.8 fontconfig-2.2.3,1 freetype2-2
	.1.7_3 gail-1.8.1 gettext-0.13.1_1 glib-2.4.8 gnomehier-1.0_20 gtk-2.4.14 hicol
	or-icon-theme-0.5 imake-6.7.0_2 jpeg-6b_3 libIDL-0.8.4 libXft-2.1.6 libart_lgpl
	2-2.3.16 libbonobo-2.8.0_1 libglade2-2.4.1 libgnomecanvas-2.8.0 libiconv-1.9.2_
	1 libxml2-2.6.16 pango-1.6.0 perl-5.8.5 pkgconfig-0.15.0_1 png-1.2.7 popt-1.7 p
	ython-2.4 shared-mime-info-0.15_1 tiff-3.7.0_2 xmlcatmgr-2.1 xorg-fonts-encodin
	gs-6.7.0 xorg-fonts-truetype-6.7.0 xorg-libraries-6.7.0_3|http://developer.gnom
	e.org/projects/gap/|||
	  */

	 /*
	  * setName("at-spi")
	  * 1.6.1
	  * /usr/ports/accessibility/at-spi
	  */
	 pkg->name = QStringList::split('/', splitLine[1]).last();
	 pkg->latestVersion = QStringList::split('-', splitLine[0]).last();
	 /* Not used in UI
	  *
	 pkg->portPath = splitLine[1];
	  */


	 /*
	  * An Assistive Technology Service Provider Interface
	  */
	 pkg->shortDescr = splitLine[3];


	 /* (Not used in UI)
	  *
	  * gnome@FreeBSD.org
	  *
	 pkg->mail = splitLine[5];
	  */


	 /*
	  * accessibility x11-toolkits
	  */
	 pkg->categories = QStringList::split(' ', splitLine[6]);


	 /*
	  * http://www.gnome.org/
	  */
	 pkg->website = splitLine[9];

	 return  pkg;
}


/** @return The "ID" of the package, category/name.
  *			e.g. "multimedia/mplayer"
  */
QString	Package::getID()	const {
	return  getMainCategory() + '/' + getName();
}


/** Returns TRUE if the installed size of the package is
  * set, FALSE if the package is not installed or
  * the size would be calculated if getInstalledSize()
  * is called.
  */
bool	Package::isInstalledSizeSet() {
	return installedSizeSet;
}


/** Returns the installed size of the package if it is
  * installed on the system, otherwise 0.
  */
Size	Package::getInstalledSize() {
	if(installedSizeSet)
		return installedSize;

	if(instStatus != Installed && instStatus != InstalledOutdated && instStatus != InstalledNewer)
		return Size();

	int  size = Executor::intFromCmd(QString("/usr/sbin/pkg_info -qs '%1'").arg(installedName));
	installedSize = Size(size);
	installedSizeSet = true;
	return installedSize;
}


bool  Package::isRequiredBy(const Package & pkg) {
	return requiredBy.contains(pkg.installedName);
}


QString Package::getLongDescr() {
	if(longDescrSet)
		return longDescr;

	QFile  f(PORTSDIR + getID() + "/pkg-descr");
	if(f.open(IO_ReadOnly)) {
		QTextStream s(&f);
		longDescr = s.read();
		f.close();
	} else
		longDescr = "";

	longDescrSet = true;
	return longDescr;
}

==> desktopbsd-tools-1.6.tar.bz2: desktopbsd-tools/libdesktopbsd/package.cpp <==
/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Authors: Peter Hofer <ph@desktopbsd.net>
 *			 Reinhard Pointner
 *			 (C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include "package.h"

#include <dirent.h>

#include <qfile.h>
#include <qregexp.h>

#include <pkgversion.h>
#include <executor.h>
#include <pkgconstants.h>

Package::Package() {
	instStatus = Undefined;
	installedSizeSet = false;
	longDescrSet = false;
}

/** This method expects *this to be the newest available
  * package (e.g. an object created from an INDEX 5 line)
  * and the parameter to be an object with the installed
  * package's information (from /var/db/pkg).
  */
void	Package::mergeInstalledPackage(const Package & pkg) {
	installedName = pkg.installedName;
	installedVersion = pkg.installedVersion;
	installedSizeSet = pkg.installedSizeSet;
	if(installedSizeSet)
		installedSize = pkg.installedSize;
	requiredBy = pkg.requiredBy;

	int status = version_cmp(getInstalledVersion(), getLatestVersion());
	instStatus = (status == -1) ? InstalledOutdated : ((status == 0) ? Installed : InstalledNewer);
}

/** Returns a list with all packages currently installed
  * on the system.
  */
QPtrList<Package>	Package::getInstalledPackages() {
	QPtrList<Package>	list;

	Package	*pkg;
	DIR		*dir = opendir(PKGDB_DIRECTORY);

	if(dir != NULL) {
		struct dirent	*ent;
		while((ent = readdir(dir)) != NULL)
			if(ent->d_type == DT_DIR  &&  ent->d_name[0] != '.') {
				pkg = new Package();
				pkg->instStatus = Installed;
				pkg->installedName = ent->d_name;			/* e.g. cups-base-1.1.23.0_3	*/

				/* Package origin not found in +CONTENTS */
				int versionpos = pkg->installedName.findRev('-', -2);
				pkg->installedVersion = pkg->installedName.mid(versionpos + 1);

				char  buffer[4096];
				char  fnbuffer[MAXNAMLEN] = { PKGDB_DIRECTORY };
				char  *end = stpcpy(fnbuffer + sizeof(PKGDB_DIRECTORY) - 1, ent->d_name);
				FILE  *fp;

#if 0
				/* Deactivate, it's not required and almost always
				 * merged with the package from the INDEX file 
				 */

				strcpy(end, "/+COMMENT");
				fp = fopen(fnbuffer, "r");
				if(fp != NULL) {
					if(fgets(buffer, sizeof buffer, fp) != NULL)
						pkg->shortDescr = buffer;

					fclose(fp);
				}
#endif

				strcpy(end, "/+CONTENTS");
				fp = fopen(fnbuffer, "r");
				if(fp != NULL) {
					/* Note that we can only get the main category
					 * (by reading the ORIGIN comment)
					 */
					const char origin[] = { "@comment ORIGIN:" };
					char  *originBgn = buffer + sizeof origin - 1;
					while(fgets(buffer, sizeof buffer, fp) != NULL) {
						if(strncmp(buffer, origin, sizeof origin - 1) == 0) {
							char  *slash = strchr(originBgn, '/');
							if(slash != NULL) {
								*slash = '\0';
								pkg->mainCategory = originBgn;

								/* Remove '\n' retained by fgets() */
								char  *newline = strchr(slash + 1, '\n');
								if(newline != NULL)
									*newline = '\0';

								pkg->name = slash + 1;
							}

							break;
						}
					}

					fclose(fp);
				}
				
				if(pkg->name.isEmpty())
					/* Origin not found in +CONTENTS */
					pkg->name = pkg->installedName.left(versionpos);

#if 0
				/* Deactivate, it's not required and almost always
				 * merged with the package from the INDEX file 
				 */

				strcpy(end, "/+DESC");
				fp = fopen(fnbuffer, "r");
				if(fp != NULL) {
					const char www[] = { "WWW: " };

					while(fgets(buffer, sizeof buffer, fp) != NULL)
						if(strncmp(buffer, www, sizeof www - 1) == 0) {
							char  *wwwStart = buffer + sizeof www - 1;
							while(*wwwStart == ' ')
								++wwwStart;

							if(*wwwStart != '\n'  &&  *wwwStart != '\0') {
								/* Remove '\n' retained by fgets() */
								char  *newline = strchr(buffer, '\n');
								if(newline != NULL)
									*newline = '\0';

								pkg->website = wwwStart;
							}

							break;
						}
					fclose(fp);
				}
#endif

				strcpy(end, "/+REQUIRED_BY");
				fp = fopen(fnbuffer, "r");
				if(fp != NULL) {
					while(fgets(buffer, sizeof buffer, fp) != NULL) {
						/* Remove '\n' retained by fgets() */
						char  *newline = strchr(buffer, '\n');
						if(newline != NULL)
							*newline = '\0';

						pkg->requiredBy.append(buffer);
					}

					fclose(fp);
				}

				list.append(pkg);
			}
			
		  closedir(dir);
	}

	return list;
}


/** Creates a "Package" object from an INDEX-5 line. The version
  * number from the file will be written into latestVersion.
  *
  * This function will not set requiredBy.
  */
Package  *Package::fromINDEXLine(const QString  & line) {
	 Package  *pkg = new Package();
	 
	 /* Assume this package is not installed - merging with
	  * a package object created in getInstalledPackages()
	  * sets this to the appropriate value anyway.
	  */
	 pkg->instStatus = NotInstalled;
	 
	 QStringList  splitLine = QStringList::split('|', line, true);

	 /* Example:
	desktopbsd-tools-1.1|/usr/ports/sysutils/desktopbsd-tools|/usr/local|Deskto
	pBSD tools for easy system configuration|/usr/ports/sysutils/desktopbsd-too
	ls/pkg-descr|ph@desktopbsd.net|sysutils||||/usr/ports/devel/qmake /usr/port
	s/x11-toolkits/qt33 /usr/ports/x11/kdebase3 /usr/ports/x11/kdelibs3|/usr/po
	rts/shells/bash /usr/ports/sysutils/fastest_cvsup /usr/ports/sysutils/portu
	pgrade /usr/ports/x11-toolkits/qt33 /usr/ports/x11/kdebase3 /usr/ports/x11/
	kdelibs3 /usr/ports/x11/xterm|http://desktopbsd.net
	  */

	 /*
	  * pkg->name = "desktopbsd-tools"
	  * pkg->latestVersion = "1.1"
	  */
	 pkg->name = splitLine[1].section('/', -1);
	 pkg->mainCategory = splitLine[1].section('/', -2, -2);
	 
	 int versionpos = splitLine[0].findRev('-', -2);
	 pkg->latestVersion = splitLine[0].mid(versionpos + 1);


	 /* "DesktopBSD tools for easy system configuration" */
	 pkg->shortDescr = splitLine[3];


	 /* (Not used in UI)
	  *
	  * ph@desktopbsd.net
	  *
	 pkg->mail = splitLine[5];

	  *
	  * { "sysutils" }
	  
	 pkg->categories = QStringList::split(' ', splitLine[6]);
	  */


	 /* "http://desktopbsd.net" */
	 pkg->website = splitLine[9];

	 return  pkg;
}


/** @return The "ID" of the package, category/name.
  *			e.g. "multimedia/mplayer"
  */
QString	Package::getID()	const {
	return  getMainCategory() + '/' + getName();
}


/** Returns TRUE if the installed size of the package is
  * set, FALSE if the package is not installed or
  * the size would be calculated if getInstalledSize()
  * is called.
  */
bool	Package::isInstalledSizeSet() {
	return installedSizeSet;
}


/** Returns the installed size of the package if it is
  * installed on the system, otherwise 0.
  */
Size	Package::getInstalledSize() {
	if(installedSizeSet)
		return installedSize;

	if(instStatus != Installed && instStatus != InstalledOutdated && instStatus != InstalledNewer)
		return Size();

	int  size = Executor::intFromCmd(QString("/usr/sbin/pkg_info -qs '%1'").arg(installedName));
	installedSize = Size(size);
	installedSizeSet = true;
	return installedSize;
}


bool  Package::isRequiredBy(const Package & pkg) {
	return requiredBy.contains(pkg.installedName);
}


QString Package::getLongDescr() {
	if(longDescrSet)
		return longDescr;

	QFile  f(PORTS_DIRECTORY + getID() + "/pkg-descr");
	if(f.open(IO_ReadOnly)) {
		QTextStream s(&f);
		longDescr = s.read();
		f.close();
	} else
		longDescr = "";

	longDescrSet = true;
	return longDescr;
}
